/*  open2300 - vws_fetch2300.c
 *  
 *  Version 1.11
 *  
 *  Control WS2300 weather station
 *  
 *  Copyright 2003-2005, Kenneth Lavrsen
 *  This program is published under the GNU General Public license
 */

#include "rw2300.h"

 
/********** MAIN PROGRAM ************************************************
 *
 * This program reads selective data from a WS2300 weather station and
 * writes it to standard out. This program was cloned from fetch2300.c 
 * and modified for use with the Virtual Weather Station (VWS) software.
 * This program will query the ws2300 weatherstation only for the data
 * elements needed by VWS.  In order to further reduce the load on the 
 * ws2300 weatherstation, data is written out to a save file.  This
 * allows for more frequent wind updates and less frequent updates for
 * other data elements.
 *
 * It takes one parameter which is the config file name with path
 * If this parameter is omitted the program will look at the default paths
 * See the open2300.conf-dist file for info
 *
 ***********************************************************************/
int main(int argc, char *argv[])
{
	WEATHERSTATION ws2300;
	unsigned char logline[3000] = "";
	unsigned char logline_save[3000] = "";
	char datestring[50];     //used to hold the date stamp for the log file
	char datestring_save[50];     //used to hold the date stamp for the log file
	FILE *fptr_measurement;

	double winddegrees;
	double ti, to, dp, ws, wc, rp;
	double r1h, r24h, rtot;
	int rhi, rho;
	char tendency[15];
	char forecast[15];
	struct config_type config;
	struct save_data_type save_data;
	time_t basictime;

	get_configuration(&config, argv[1]);

	read_saved_data(&save_data);


	ws2300 = open_weatherstation(config.serial_device_name);

	/* READ TEMPERATURE INDOOR */
	
	if (save_data.ti_present)
		ti = save_data.ti;
	else
		ti = temperature_indoor(ws2300, config.temperature_conv);
	sprintf(logline, "%sTi %.1f\n", logline, ti);
	sprintf(logline_save, "%sTi %.1f\n", logline_save, ti);

	
	
	/* READ TEMPERATURE OUTDOOR */

	if (save_data.to_present)
		to = save_data.to;
	else
		to = temperature_outdoor(ws2300, config.temperature_conv);
	sprintf(logline, "%sTo %.1f\n", logline, to);
	sprintf(logline_save, "%sTo %.1f\n", logline_save, to);

	

	/* READ DEWPOINT */

	if (save_data.dp_present)
		dp = save_data.dp;
	else
		dp = dewpoint(ws2300, config.temperature_conv);
	sprintf(logline, "%sDP %.1f\n", logline, dp);
	sprintf(logline_save, "%sDP %.1f\n", logline_save, dp);
	
	
	/* READ RELATIVE HUMIDITY INDOOR */

	if (save_data.rhi_present)
		rhi = save_data.rhi;
	else
		rhi = humidity_indoor(ws2300);
	sprintf(logline, "%sRHi %d\n", logline, rhi );
	sprintf(logline_save, "%sRHi %d\n", logline_save, rhi );
	
	
	/* READ RELATIVE HUMIDITY OUTDOOR */

	if (save_data.rho_present)
		rho = save_data.rho;
	else
		rho = humidity_outdoor(ws2300);
	sprintf(logline, "%sRHo %d\n", logline, rho );
	sprintf(logline_save, "%sRHo %d\n", logline_save, rho );


	/* READ WIND SPEED AND DIRECTION */
	
	if (wind_current(ws2300, config.wind_speed_conv_factor, &ws, &winddegrees) == 0)  {
		ws = save_data.ws;
		winddegrees = save_data.dir0;
	}
	sprintf(logline,"%sWS %.1f\n", logline, ws);
	sprintf(logline_save,"%sWS %.1f\n", logline_save, ws);
       	sprintf(logline, "%sDIR0 %.1f\n", logline, winddegrees);
        sprintf(logline_save, "%sDIR0 %.1f\n", logline_save, winddegrees);

			
			
	/* WINDCHILL */
	
	if (save_data.wc_present)
		wc = save_data.wc;
	else
		wc = windchill(ws2300, config.temperature_conv);
	sprintf(logline, "%sWC %.1f\n", logline, wc);
	sprintf(logline_save, "%sWC %.1f\n", logline_save, wc);
	        
	
	/* READ RAIN 1H */

	if (save_data.r1h_present)
		r1h = save_data.r1h;
	else
		r1h = rain_1h(ws2300, config.rain_conv_factor);
	sprintf(logline,"%sR1h %.2f\n", logline, r1h);
	sprintf(logline_save,"%sR1h %.2f\n", logline_save, r1h);



	/* READ RAIN 24H */

	if (save_data.r24h_present)
		r24h = save_data.r24h;
	else
		r24h = rain_24h(ws2300, config.rain_conv_factor);
	sprintf(logline,"%sR24h %.2f\n", logline, r24h);
	sprintf(logline_save,"%sR24h %.2f\n", logline_save, r24h);

	
	/* READ RAIN TOTAL */
	
	if (save_data.rtot_present)
		rtot = save_data.rtot;
	else
		rtot = rain_total(ws2300, config.rain_conv_factor);
	sprintf(logline,"%sRtot %.2f\n", logline, rtot);
	sprintf(logline_save,"%sRtot %.2f\n", logline_save, rtot);


	/* READ RELATIVE PRESSURE */

	if (save_data.rp_present)
		rp = save_data.rp;
	else
		rp = rel_pressure(ws2300, config.pressure_conv_factor);
	sprintf(logline,"%sRP %.3f\n", logline, rp);
	sprintf(logline_save,"%sRP %.3f\n", logline_save, rp);


	/* READ TENDENCY AND FORECAST */
	
	if (save_data.tendency_present && save_data.forecast_present)  {
		strcpy(tendency, save_data.tendency);
		strcpy(forecast, save_data.forecast);
	}
	else
		tendency_forecast(ws2300, tendency, forecast);
	sprintf(logline, "%sTendency %s\nForecast %s\n", logline, tendency, forecast);
	sprintf(logline_save, "%sTendency %s\nForecast %s\n", logline_save, tendency, forecast);

	
	if (save_data.update_bat == 1 && save_data.fetch_delay > 0)  {
		if (write_init2300_file(save_data.fetch_delay))
			save_data.update_bat = 0;
	}
	else if (read_init2300_file() == 0)
		save_data.update_bat = 1;
	sprintf(logline_save, "%sUpdateBAT %d\n", logline_save, save_data.update_bat);
	sprintf(logline_save, "%sUpdateSeconds %d\n", logline_save, save_data.update_seconds);
	sprintf(logline_save, "%sFetchDelay %d\n", logline_save, save_data.fetch_delay);

	close_weatherstation(ws2300);

	/* GET DATE AND TIME FOR LOG FILE, PLACE BEFORE ALL DATA IN LOG LINE */
	
	time(&basictime);
	if (save_data.use_datetime)
		strftime(datestring_save,sizeof(datestring_save),"Date %Y-%b-%d\nTime %H:%M:%S\n",
	         	 localtime(&save_data.datetime));
	else
		strftime(datestring_save,sizeof(datestring_save),"Date %Y-%b-%d\nTime %H:%M:%S\n",
	         	 localtime(&basictime));
	strftime(datestring,sizeof(datestring),"Date %Y-%b-%d\nTime %H:%M:%S\n",
	       	 localtime(&basictime));

	// Print out and leave

	printf("%s%s",datestring, logline);

	strftime(datestring,sizeof(datestring),"WDate %Y-%b-%d\nWTime %H:%M:%S\n",
	       	 localtime(&basictime));

	if ((fptr_measurement = fopen("./Measurement Save.txt", "w")) != NULL)  {
		fprintf(fptr_measurement, "%s%s%s", datestring_save, datestring, logline_save);
		fclose(fptr_measurement);
	}


	return(0);
}

